/**************************************************
 *
 *   Contains the vocabulary and responses of
 *   Adventures on Planet Smir III, 1
 *
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <errno.h>
#include <ctype.h>

#include "aconst.h"
#include "words.h"
#include "stores.h"
#include "rooms.h"
#include "player.h"
#include "input.h"


#ifdef __CBM__
#include "simpleio.h"
#elif defined __ATARI__
#include "atasimpleio.h"
#endif

//Include memory-managing headers and define macros to
//minimize source code.
#ifdef __USEFARMEM__
#ifdef __C64__
#include "c64hide.h"
#elif defined __C128__
#include "membank128.h"
#define hidereadb bank1_readbyte
#define hidereadw bank1_readword
#elif defined __APPLE2ENH__

#elif defined __ATARI__
#include "memxatari.h"
#define hidereadb aux_readbyte
#define hidereadw aux_readword
#endif
#else
#define hidereadb
#define hidereadw
#endif


/*************************************
	Items
 *************************************/

#ifdef __C128__
#ifdef __USEFARMEM__
#pragma bss-name (push, "B1BSS")
#else 
#pragma bss-name (push, "APPBSS")
#endif
#elif defined __C64__
#pragma bss-name (push, "V2BSS")
#elif defined __PLUS4__
#pragma bss-name (push, "TAPEBSS")
#elif defined __ATARI__
#pragma bss-name (push, "TBUFBSS")
#endif

#pragma local-strings (on)

//Pointer to current alias being scanned.  Used to optimize
//alias scanning.
extern struct item_Alias* ita;
#pragma zpsym ("ita")

//Current item #.
extern unsigned char Itm1;
#pragma zpsym ("Itm1")

//Current room #.
extern unsigned char CRoom;
#pragma zpsym ("CRoom")

//Temp. variables.
extern unsigned char i, j, e, c;
#pragma zpsym ("i")
#pragma zpsym ("j")
#pragma zpsym ("e")
#pragma zpsym ("c")

extern char* s;
#pragma zpsym ("s")

//Current pos. in user input.
extern unsigned char CurPos;
#pragma zpsym ("CurPos")

#ifdef __ATARI__
#define printh aux_print
#elif defined __C128__
#define printh bank1_print
#endif

extern unsigned char c;
#pragma zpsym ("c")

//Contains the player's current command.
extern char Input [38];

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#elif defined __C64__
#pragma code-name (push, "BAS2CODE")
#elif defined __PLUS4__
#pragma code-name (push, "TAPECODE")
#elif defined __ATARI__
#pragma code-name (push, "MBUFCODE")
#endif
#ifndef __C128__
#pragma code-name (pop)
#endif

//Current item duruing item scan.
extern unsigned char CurItem;
#pragma zpsym ("CurItem")

#ifdef __C128__
#pragma rodata-name (push, "TAPECONST")
#endif

const char* Message []=
{

/*  Message 0  */
	"    \"Sorry.  I don't have that.\"",
       /*0        1         2         3         */
       /*123456789012345678901234567890123456789*/

/*  Message 1  */
	"You have too many items.  You can only\n"
	"hold a maximum of eight items.",

/*  Message 2  */
	"You can't drop that here.  A room can\n"
	"only contain a maximum of eight items,\n"
	"and this room is full.",

/*  Message 3  */
	"You already have that.",

/*  Message 4  */
	"You can't go that way.",

/*  Message 5  */
	"That item is not available.",

/*  Message 6  */
	"Please be more specific.",
};

#ifdef __C128__
#pragma rodata-name (pop)
#endif

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#endif

#ifdef __C64__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __PLUS4__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __ATARI__
#pragma code-name (push, "TBUFCODE")
#endif

//Print message for given #.
void __fastcall__ printmsg(unsigned char m)
{printscr ((void*)Message[m]);}

//Middleman to add 15 to player score.  You can define others.
void addsc15 (void)
{Player.Score+=15;}

//Print item not found message.
void printnoitem (void)
{printmsg(msgItemNotAvail); }

void printokay (void)
{printscr ("Okay.");}

extern struct room* CRm;
#pragma zpsym ("CRm")

//Player.RoomInv[CRoom]
extern unsigned char* CurRoomInv;
#pragma zpsym ("CurRoomInv");

#ifdef __PLUS4__
#pragma code-name ("BASICODE")
#endif

#ifdef __C128__
#pragma code-name ("APPCODE")
#endif

#ifdef __C64__
#pragma code-name ("BAS2CODE")
#endif

#ifdef __ATARI__
#pragma code-name ("MBUFCODE")
#endif

//Prints a period followed by a return.
void __fastcall__ printperiod (void)
{printscr (".");}

//Prints the name of the current item indicated by Itm1.
static void __fastcall__ printitmname (void)
{
#ifdef __USEFARMEM__
#ifdef __C128__
	//bank1_print (bank1_readword((void*)&Item[Itm1].Name));
	prints (Item[Itm1].Name);
#elif defined __C64__
	printh (hidereadw((void*)&Item[/*CurRoomInv[i]*/Itm1].Name));
#elif defined __ATARI__
	aux_print (aux_readword((void*)&Item[Itm1].Name));
#endif
#else
	prints (Item[Itm1].Name);
#endif
}

#ifdef __ATARI__
#pragma code-name ("MBUFCODE")
#endif

#ifdef __C64__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

extern char* inx;
#pragma zpsym ("inx")

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#elif defined __C64__
#pragma code-name (push, "BASINCODE")
#elif defined __PLUS4__
#pragma code-name (push, "CODE")
#elif defined __ATARI__
#pragma code-name (push, "CODE")
#endif
#if defined __USEFARMEM__ && !defined __C128__
//Compares player noun to given word.
//Returns 1 if match an 0 otherwise.
static unsigned char __fastcall__ ScanWord (char* Dest)
{
	//static unsigned char /*CurPos,*/ c;
	//Dest=hidereadw((void*)&Dest);
//printu (Dest); //printcr ();
//printf ("0x%X, ",Dest);
//printh (Dest); printcr ();
	// for (j=0;
		// //(c=hidereadb((char*)&Dest[j])) == Sour[j] && Sour[j]!=0;
		// (hidereadb((char*)&Dest[j])) == (c=inx[j]) && c/*Sour[j]*/;
		// ++j);
	/*if (c == 0) {
		return 1;
	}
	return 0;*/
//printcr ();
	__asm__ (
		"\tsta\tptr4\n"
		"\tstx\tptr4+1\n"
		"\tldy\t#0\n"
		"\tsty\ttmp3\n"
		"@a01:\n"
		"\tldy\ttmp3\n"
		"\tlda\t(%v),y\n"
		"\tsta\t_c\n"
		//"\tcmp\t(ptr1),y\n"
		"\tlda\tptr4\n"
		"\tldx\tptr4+1\n"
		"\tjsr\t%v\n"
		"\tinc\ttmp3\n"
		"\tinc\tptr4\n"
		"\tbne\t@a02\n"
		"\tinc\tptr4+1\n"
		//"\tbne\t@a01\n"
		"@a02:\n"
		"\tcmp\t_c\n"
		"\tbne\t@a09\n"
		"\tlda\t_c\n"
		"\tbne\t@a01\n"
		// "\tlda\t_c\n"
		// "\tbne\t@a01\n"
		"@a09:\n",
		inx, hidereadb
		);
	return c==0;
/*	static char* OldDest;
	static char c;
	OldDest=Dest;
	while	((c=hidereadb(Dest)) == *Sour && *Sour!=0) {++Dest; ++Sour;}
	if (!c) {*len=Dest-OldDest; return 1;}
	return 0;*/
}

#ifdef __PLUS4__
#pragma code-name (pop)
#endif
#ifdef __C64__
#pragma code-name (pop)
#endif
#ifdef __ATARI__
//#pragma code-name (pop)
#pragma code-name ("MBUFCODE")
#endif

#ifdef __C128__
#pragma code-name (push, "TAPECODE")
#endif

//Searches for the verb given at the start of Input and returns the verb #.
//Returns 0xFF if not found.
unsigned char FindVerb (void)
{
	//static unsigned char c;
	j=0;
	for (; j < NumVerbs; ++j)
	{
//printscr (Verb[j].Name);
		for (CurPos=0;
#ifdef __C128__
			Input[CurPos] == (c=(Verb[j].Name[CurPos])) &&
#else
			(c=(hidereadb((char*)hidereadw((void*)&Verb[j].Name)+CurPos))) && 
			Input[CurPos] == c &&
#endif
			Input[CurPos] >= 33 && c
				; ++CurPos) ;
			if (Input[CurPos]<33 && c==0)
			{
				//CurPos=CurPos;
				return j;
			}
	}
//printc (',');
	for (; Input[CurPos]>=33; ++CurPos);
	Input[CurPos]=0;
	return -1;
}
#ifdef __C128__
#pragma code-name (pop)
#endif

#else
	
#ifdef __C128__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __C64__
#pragma code-name (push, "BASINCODE")
#endif


//Compares the given word to the input noun.
//Returns 1 if equal and 0 if not.
static unsigned char __fastcall__ ScanWord (char* Dest)
{
	//static unsigned char /*CurPos,*/ c;
	//Dest=hidereadw((void*)&Dest);
//printu (Dest); //printcr ();
//printf ("0x%X, ",Dest);
//printh (Dest); printcr ();
//printscr (Dest);
	// for (j=0;
		// //(c=hidereadb((char*)&Dest[j])) == Sour[j] && Sour[j]!=0;
		// Dest[j] == (c=inx[j]) && c/*Sour[j]*/;
		// ++j);
	/*if (c == 0) {
		return 1;
	}
	return 0;*/
	__asm__ (
		"\tsta\tptr1\n"
		"\tstx\tptr1+1\n"
		"\tldy\t#0\n"
		"@a01:\n"
		"\tlda\t(%v),y\n"
		"\tsta\t_c\n"
		"\tcmp\t(ptr1),y\n"
		"\tbne\t@a09\n"
		"\tiny\n"
		"\tlda\t_c\n"
		"\tbne\t@a01\n"
		"@a09:\n",
		inx
		);
//printcr ();
	return c==0;
/*	static char* OldDest;
	static char c;
	OldDest=Dest;
	while	((c=hidereadb(Dest)) == *Sour && *Sour!=0) {++Dest; ++Sour;}
	if (!c) {*len=Dest-OldDest; return 1;}
	return 0;*/
}

#ifdef __C128__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

unsigned char FindVerb (/*char* Input*/void)
{
	char d;
	j=0;
	for (j=0; j < NumVerbs; ++j)
	{
		for (i=0;
			(d=Input[i]) == (c=(Verb[j].Name[i])) &&
			d >= 33 && c
				; ++i);
		if (d<33)
		{
			CurPos=i;
			return j;
		}
	}
	for (; Input[i]>=33; ++i);
	Input[i]=0;
	return -1;
}
#endif

#if !defined __C64__ && !defined __PLUS4__

#pragma code-name (pop)
#endif

// static unsigned char __fastcall__ ScanWord (char* x)
// {
	// return ScanWord(x, inx);
// }

#if defined __USEFARMEM__ && !defined __C128__
#ifdef __C128__
#pragma code-name (push, "APPCODE")
#endif

#if defined __ATARI__ && !defined __USEFARMEM__
#pragma code-name (pop)
#endif
//Gets the current location in user input and scans for a supported noun.
//Returns the item #.
#ifdef __ATARI__
#pragma code-name (pop)
#pragma code-name (push, "MBUFCODE")
#endif

//Scan items for user input.
unsigned char __fastcall__ FindItem (void)
{
	inx=&Input[CurPos];
/*  Search for item in Input */
	for (CurItem=0; CurItem < NumItems; ++CurItem)
	{
		if (ScanWord(hidereadw((void*)&Item[CurItem].Name))) {
			return CurItem;
		}
	}
/*  Search item aliases */
	for (CurItem=0; CurItem < NumItemAliases; ++CurItem)
	{
		if (ScanWord(hidereadw((void*)(ita=&ItemAlias[CurItem])->Name))) {
			return hidereadb((void*)&ita->Num);
		}
	}

/*  Search item categories */
	for (CurItem=0; CurItem < NumItemCate; ++CurItem)
	{
		if (ScanWord(hidereadw((void*)&ItemCate[CurItem].Name))) {
			return CurItem|128;
		}
	}

/*  Search directions  */
	for (CurItem = 0; CurItem < 4; ++CurItem)
	{
		if (ScanWord((char*)&RoomDir[CurItem])) {
			return CurItem | 160;
		}
	}
	return -1;
}

#else
unsigned char __fastcall__ FindItem (void)
{
	inx=&Input[CurPos];
//printscr (inx);

/*  Search for item in Input */
	for (CurItem=0; CurItem < NumItems; ++CurItem)
	{
		if (ScanWord(Item[CurItem].Name)) {
			return CurItem;
		}
	}
/*  Search item aliases */
	for (CurItem=0; CurItem < NumItemAliases; ++CurItem)
	{
		if (ScanWord((ita=&ItemAlias[CurItem])->Name)) {
			return ita->Num;
		}
	}

/*  Search item categories */
	for (CurItem=0; CurItem < NumItemCate; ++CurItem)
	{
		if (ScanWord(ItemCate[CurItem].Name)) {
			return CurItem|128;
		}
	}
/*  Search directions  */
	for (CurItem = 0; CurItem < 4; ++CurItem)
	{
		if (ScanWord((char*)&RoomDir[CurItem])) {
			return CurItem | 160;
		}
	}
	return -1;
}
#endif
#if defined __ATARI__ && !defined __USEFARMEM__
#pragma code-name (pop)
#endif

#ifdef __C128__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
//#pragma code-name (pop)
#endif

//#endif

//#pragma code-name (push, "BAS1CODE")
#ifdef __C128__
#pragma code-name ("APPCODE")
#elif defined __C64__
#pragma code-name (push, "CODE")
#elif defined __PLUS4__
#pragma code-name (push, "BASICODE")
#elif defined __ATARI__
//#pragma code-name (push, "TBUFCODE")

#endif

#ifdef __C64__
#pragma code-name (push, "BAS2CODE")
#endif

#ifdef __ATARI__
#pragma code-name (push, "TBUFCODE")
#endif

//Adds the item # in Itm1 to the given inventory pointer.
//Returns 0 if successful and 1 if full.
unsigned char __fastcall__ AddInv (unsigned char* Inv)
{
	//static unsigned char i;
	// for (i=0; i<8&&Inv[i]!=0xFF;)
		// ++i;
	// if (i==8) return 1;
	// Inv[i]=Itm1;
// //brdrcol=i; 
	// if (i<7) {
		// ++i;
		// Inv[i]=-1;
	// }
	// return 0;
	__asm__ (
		"\tsta\tptr1\n"
		"\tstx\tptr1+1\n"
		"\tldy\t#0\n"
		"@a01:\n"
		"\tlda\t(ptr1),y\n"
		"\tcmp\t#$FF\n"
		"\tbeq\t@a02\n"
		"\tiny\n"
		"\tcpy\t#8\n"
		"\tbcc\t@a01\n"
		"\tjmp\treturn1\n"
		"@a02:\n"
		"\tlda\t_Itm1\n"
		"\tsta\t(ptr1),y\n"
		"\tiny\n"
		"\tcpy\t#8\n"
		"\tbcs\t@a03\n"
		"\tlda\t#$FF\n"
		"\tsta\t(ptr1),y\n"
		"@a03:\n"
		"\tjmp\treturn0\n"
		);
};

// #ifdef __PLUS4__
// #pragma code-name (pop)
// #endif

#ifdef __PLUS4__
#pragma code-name ("TAPECODE")
#endif

unsigned char AddInvPlayer (void)
{return AddInv (Player.Inv);}
//#pragma code-name (pop)
//#pragma code-name (push, "BAS1CODE")

#ifdef __PLUS4__
#pragma code-name ("BASICODE")
#endif

#ifdef __C64__
#pragma code-name (pop)
#endif

//Removes the item given in Itm1 from the inventory given by Inv.
//Returns 1 if successful and 0 otherwid=se.
unsigned char __fastcall__ RemoveInv (unsigned char* Inv)
{
	///*register*/ static unsigned char i;// = 0;
	// for (i=0; i < 8 && Inv[i] != 0xFF; ++i)
	// {
		// if (Inv [i] == Itm1)
		// {
			// for (; i<7; ++i)
			// Inv [i] = Inv [i+1];
			// Inv [7] = -1;
			// return 1;
		// }
	// };
	// return 0;
	__asm__ (
		"\tsta\tptr1\n"
		"\tstx\tptr1+1\n"
		"\tldy\t#0\n"
		"@a01:\n"
		"\tlda\t(ptr1),y\n"
		"\tbmi\t@a09\n"
		"\tcmp\t_Itm1\n"
		"\tbeq\t@a02\n"
		"\tiny\n"
		"\tcpy\t#8\n"
		"\tbcc\t@a01\n"
		"@a09:\n"
		"\tjmp\treturn0\n"
		"@a02:\n"
		"\tiny\n"
		"\tlda\t(ptr1),y\n"
		"\tdey\n"
		"\tsta\t(ptr1),y\n"
		"\tiny\n"
		"\tcpy\t#8\n"
		"\tbcc\t@a02\n"
		"\tlda\t#$FF\n"
		"\tldy\t#7\n"
		"\tsta\t\t(ptr1),y\n"
		"\tjmp\treturn1\n"
		);
};

//#pragma code-name (push, "BAS1CODE")

/* Searches inventory at Inv for item Itm.
 * Returns 1 for found and 0 for not found.
 */
unsigned char __fastcall__ SearchInv (unsigned char* Inv, unsigned char Itm)
{
	/*register*//*static unsigned char*/ //i=0;
	// for (; i<8 && Inv[i]!=0xFF; ++i) {
		// if (Inv[i]==Itm) return 1;
	// }
	// return 0;
	__asm__ (
		"\tsta\ttmp1\n"
		"\tjsr\tpopax\n"
		"\tsta\tptr1\n"
		"\tstx\tptr1+1\n"
		"\tldy\t#0\n"
		"@a01:\n"
		"\tlda\t(ptr1),y\n"
		"\tcmp\t#$FF\n"
		"\tbeq\t@a09\n"
		"\tcmp\ttmp1\n"
		"\tbne\t@a02\n"
		"\tjmp\treturn1\n"
		"@a02:\n"
		"\tiny\n"
		"\tcpy\t#8\n"
		"\tbcc\t@a01\n"
		"@a09:\n"
		"\tjmp\treturn0\n"
		);
}

//SearchInv() middlemen.
#ifdef __C64__
#pragma code-name (push, "BASINCODE")
#endif

unsigned char __fastcall__ SearchInv2 (unsigned char* Inv)
{
	return SearchInv (Inv, Itm1);
}

unsigned char __fastcall__ SearchInvPlayer (void/*unsigned char* Inv*/)
{
	return SearchInv2 (Player.Inv);
}

unsigned char SearchInvRoom (void)
{SearchInv2 (*CurRoomInv);}
//#pragma code-name (pop)

#ifdef __C64__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
#endif

#ifdef __C128__
#pragma code-name ("APPCODE")
#endif

//Scans given inventory for an item in the category specified by Itm1.
//Returns o if found, 1 if not and 2 if more than one item found.
//Also prints error messages.
#if defined __USEFARMEM__ && !defined __C128__
unsigned char __fastcall__ GetCateItem (/*register*/ unsigned char* Inv)
{
	//i=Itm1&127;
	c=0xFF;
	if (Itm1>=128 && Itm1<160) {
		for (;
			(e=hidereadb((void*)&ItemCate[Itm1&127].Items[j]))!=0xFF;
			++j)
		{
			//if (SearchInv (Inv, hidereadb((void*)&ItemCate[i-128].Items[j])))
			if (SearchInv (Inv, e/*=hidereadb((void*)&ItemCate[i].Items[j])*/))
			{
				//if (c!=-1) {puts (Message[msgBeMoreSpec]); return 2;}
				if (c!=0xFF) {printmsg(msgBeMoreSpec); return 2;}
				//c=hidereadb((void*)&ItemCate[i].Items[j]);
				c=e;
			}
		}
		if (c==0xFF) {
			//puts (Message[msgItemNotAvail]); return 1;
			printnoitem (); return 1;//printmsg(msgItemNotAvail); return 1;
		}
		Itm1=c;
		//return 0;
	}
	return 0;
}

#else

//Searches the given inventory for items in the category given in Itm1.
//(Itm1 is from 128 to 159.)
/* Returns 0 if item found, 1 if unavailable and 2 if more than one item. */
unsigned char __fastcall__ GetCateItem (/*register*/ unsigned char* Inv)
{
	/*register *//*unsigned char*/ /*i,*/ j=0, c=-1;//, i4;
	//static unsigned char* z;
	i=Itm1&127; //c=-1, j=0;
	//if (i>=128 && i<160) {
	if (Itm1>=128 &&  Itm1<160) {
		for (;//j=0;
			//hidereadb((void*)&ItemCate[i-128].Items[j]);
			e=ItemCate[i].Items[j];
			++j)
		{
			//if (SearchInv (Inv, hidereadb((void*)&ItemCate[i-128].Items[j])))
			if (SearchInv (Inv, e/*=hidereadb((void*)&ItemCate[i].Items[j])*/))
			{
				//if (c!=-1) {puts (Message[msgBeMoreSpec]); return 2;}
				if (c!=0xFF) {printmsg(msgBeMoreSpec); return 2;}
				//c=hidereadb((void*)&ItemCate[i].Items[j]);
				c=e;
			}
		}
		if (c==0xFF) {
			//puts (Message[msgItemNotAvail]); return 1;
			printnoitem (); return 1;//printmsg(msgItemNotAvail); return 1;
		}
		Itm1=c;
		//return 0;
	}
	return 0;
}
#endif
#pragma bss-name (pop)

#ifdef __C64__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __PLUS4__
#pragma code-name (push, "GRCODE")
#endif

//Middlemen for GetCateItem().
//Passes stated inventory.
unsigned char __fastcall__ GetCateItemPlayer (void)
{return GetCateItem (Player.Inv);}

unsigned char __fastcall__ GetCateItemRoom (void)
{return GetCateItem (CurRoomInv);}

#ifdef __ATARI__
#pragma code-name (pop)
#endif

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

#ifdef __C64__
#pragma code-name (pop)
#endif

#ifdef __C128__
#pragma code-name ("APPCODE")
#endif

#ifdef __ATARI__
//#pragma code-name (pop)
#endif

void vAnswer ()//char Itm1)
{
	switch (Itm1)
	{
	}
	printscr ("You can't answer that.");
};

#ifdef __PLUS4__
#pragma code-name (push, "CODE")
#endif

#ifdef __ATARI__
#pragma code-name (push, "MBUFCODE")
#endif

//Verb to describe the current item or scene.
void vLook (void)//char Itm1)
{
	static void (*func)();
	//If an item is specified:
	if (Itm1!=0xFF) {
		//If it is available in the player inventory,
#ifdef __USEFARMEM__
		if (SearchInvPlayer () || SearchInvRoom ())
		{
			//get string address and print help text.
			if ((s=hidereadw((void*)&ItemPtr->Info))==0) {
				printscr ("No help available for that item.");
				return;
			}
			printh (s); printcr ();
#else
		if (SearchInvPlayer () || SearchInvRoom ())
		{
			//get string address and print help text.
			if ((s=ItemPtr->Info)==0) {
				printscr ("No help available for that item.");
				return;
			}
			printscr (s);
#endif
		} else {
			printnoitem ();
		}
		return;
	}
	//Otherwise,
	//get room information
	CurRoomInv=Player.RoomInv[CRoom];
	CRm=&Room[CRoom];
	//print it,
#ifdef __USEFARMEM__
	printh ((char*)hidereadw((void*)&CRm->Desc));
	//call the room handler
	func=hidereadw((void*)&CRm->RoomHandler);
#else
	printscr (CRm->Desc);
	//call the room handler
	func=(CRm->RoomHandler);
#endif
	(*func) ();
	printcr();
	//and list room items.
	for (i=0; i<8 && (Itm1=CurRoomInv[i])!=0xFF;++i) {
#ifdef __USEFARMEM__
		prints ("There is a ");
		printitmname ();
		printscr (" here.");
#else
		prints ("There is a ");
		printitmname ();
		printscr (" here.");
#endif
	}
	//if (i) printcr();
}

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

#ifdef __C64__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#endif

//Middleman for vLook and no item specified.
void vLook2 (void)
{
	Itm1=-1;
	vLook ();

}

#ifdef __ATARI__
#pragma code-name (pop)
#endif

#ifdef __ATARI__
#pragma code-name (push, "MBUFCODE")
#endif

#ifdef __C64__
#pragma code-name (pop)
#endif

//Change to different room.
#ifdef __PLUS4__
#pragma code-name (push, "TAPECODE")
#endif

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#endif

#ifdef __USEFARMEM__
void vGo (void)
{
	Itm1-=160;
	if (Itm1<4) {
		//j = Itm1&0x0F; 
		if ((i=hidereadb(&(CRm->NextRoom[Itm1])))==0xFF) {
			goto err;
		}
		CRoom = i;
		Player.ExitRoom =-1; vLook2(); return;
	}
err:	printmsg(msgCantGoThere); return;
}
#else
void vGo (void)
{
	if (Itm1>=160 && Itm1<164) {
		j = Itm1&0x0F; 
		if ((i=CRm->NextRoom[j])==0xFF) {
			goto err;
		}
		CRoom = i;
		Player.ExitRoom =-1; vLook2(); return;
	}
err:	printmsg(msgCantGoThere); return;
}
#endif

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

#ifdef __C64__
#pragma code-name (push, "BAS2CODE")
#endif

#ifdef __ATARI__
#pragma code-name ("TBUFCODE")
#endif

void vExit ()
{
	if (Player.ExitRoom==0xFF) {
		printscr ("You can't exit to nowhere.");
		return;
	}
	CRoom = Player.ExitRoom;
	Player.ExitRoom=-1;
	vLook2();
}

#ifdef __PLUS4__
#pragma code-name (push, "BASBCODE")
#endif

#ifdef __C64__
#pragma code-name ("BASINCODE")
#endif

//Middlemen for quick going (i.e. N S E W).
void vGoNorth	(void)
{
	Itm1=iNorth; vGo();
}

#ifdef __C64__
#pragma code-name (pop)
#endif

void vGoSouth	(void)
{
	Itm1=iSouth; vGo();
}

void vGoEast	(void)
{
	Itm1=iEast; vGo();
}

void vGoWest	(void)
{
	Itm1=iWest; vGo();
}

//Verb-handling for picking up an item.
void vGet (void)
{
	//char i=-1;
	//if (((signed char) Itm1)<0 && Itm1<160) {
	//if (Itm1>=160) 
	//if (Itm1>=128 && Itm1<160) {
	//if ((signed char)Itm1>=160) {
		//if (GetCateItem ((void*)CurRoomInv)) return;
		if (GetCateItemRoom ()) return;
	//}
	//if (!SearchInv (Player.RoomInv[CRoom], Itm1))
	/*if (!SearchInv2 (Player.RoomInv[CRoom]))
	{
		//puts (Message[msgCantBuyHere]); return;
		printmsg (msgItemNotAvail); return;
	}*/
	if (Player.Inv[7] != 0xFF) {
	//if (SearchInv2 ((void*)&Player.Inv)) {
		//printscr ("You don't have room for that item.");
		printmsg (msgHoldingTooManyItems);
		return;
	}
	/*if (SearchInv (Player.Inv, Itm1)) {
		//printscr ("You already have that item."); return;
		printmsg (msgAlreadyHave); return;
	}*/
//printu (Itm1); printcr ();
#ifdef __USEFARMEM__
	if (/*((signed char)Itm1)>=0 &&*/ hidereadb((void*)&ItemPtr->Attrib) & itemAMovable) {
#else
	if (/*((signed char)Itm1)>=0 &&*/ ItemPtr->Attrib & itemAMovable) {
#endif
		if (!RemoveInv (CurRoomInv)) {
			//puts (Message[msgItemNotAvail]);
			printnoitem ();//printmsg(msgItemNotAvail);
			return;
		}
		/*if (AddInv (Player.Inv, Itm1) <0) {
			//printscr ("You cannot carry any more items.");
			printmsg(msgHoldingTooManyItems);
			return;
		}*/
//printu (Player.Inv); getkey ();

		//AddInv (Player.Inv);
		AddInvPlayer ();
		//printscr ("Okay.");
		printokay ();
		return;
	} //else {
		printscr ("You can't move this item."); //return;
	//}
}

#ifdef __PLUS4__
#pragma code-name (pop)
#endif

void vPut ()//char Itm1)
{
	//unsigned char i;
	//if (Itm1>=128 && Itm1<160) {
	//if ((signed char)Itm1<160) {
		if (GetCateItemPlayer ()) return;
	//}
	//i = SearchInv (Player.Inv, Itm1);
	//if (i == 0) {
	//if (SearchInv (Player.Inv, Itm1)) {
	//if (!SearchInv2 ((void*)&Player.Inv)) {
	if (!SearchInvPlayer ()) {
		//printscr ("You don't have that item."); return;
		printmsg (msgDontHave); return;
	}
	/*for (i = 0;
		Player.RoomInv [CRoom] [i] !=0xFF && i<8; ++i);
	if (i == 8) {
		printscr ("This room is full.");
		return;
	}*/
#ifdef __USEFARMEM__
	if (((signed char)Itm1)>=0 && (hidereadb((void*)&ItemPtr->Attrib) & itemAMovable)) {
#else
	if ((signed char)Itm1>=0 && ItemPtr->Attrib & itemAMovable) {
#endif
		if (!RemoveInv (Player.Inv)) {
			printnoitem ();
			return;
		}
		if (AddInv (CurRoomInv))
		{
			printmsg (msgTooManyItemsInRoom);
			return;
		}
		printokay ();
		return;
	}
	printscr ("You can't move this item.");
}

#ifdef __PLUS4__
#pragma code-name (push, "GRCODE")
#endif

//Lists player's inventory.
void vItems ()//char Itm1)
{
	printscr ("You have:");
	for (i=0; i<8 && (Itm1=Player.Inv[i])!=0xFF;) {
		printitmname (); printcr ();
		++i;
	}
	if (!i) printscr ("Nothing.");
}

#ifdef __ATARI__
#pragma code-name (pop)
#endif

#ifdef __C128__
#pragma code-name (pop)
#pragma code-name (pop)
#endif

//Displays help for current cndition.
void vHelp (void) {
	switch (CRoom) {
	}
nohelp:
	printscr ("No help currently available.");
}

extern const char* HelpPages[2];

#ifdef __PLUS4__
#pragma code-name ("CODE")
#endif

#ifdef __C128__
#pragma code-name (push, "APPCODE")
#endif

//Describes the game.
void vIntro ()
{
	//static unsigned char i;
	//clrscr();
		/*0        1         2         3         */
		/*123456789012345678901234567890123456789*/
	/*printf ("\nThe object of this game is to find your\n"
		"friend Talnon, all who were kidnapped\n"
		"and all stolen property.  I will tell no"
		"more about the object of this game.  To\n"
		"play, you type in commands on your\n"
		"computer or emulator.  Such commands\n"
		"include:\n"
		"\n"
		"get/put <object>   inventory (or i)\n"
		"go <dir>           exit\n"
		"look [<item>]      read <item>\n"
		"\n"
		"Some rooms (i.e. teleporters) require\n"
		"you to type \"enter <room>\" to enter\n"
		"them.\n"
		"\nPress Return to continue or x to return\n"
		"to game.\n");*/
#ifdef __USEFARMEM__
	printh (hidereadw((void*)&HelpPages[0]));
	//i=getkey();
	//if ((i&0x7F)=='x'/* || i=='X'*/) return;
	if ((getkey())=='x'/* || i=='X'*/) return;
	//clrscr();
		/*0        1         2         3         */
		/*123456789012345678901234567890123456789*/
	/*printf ("\nSpecial commands are also available:\n"
		"\n"
		"enter <room type> to enter teleporters &"
		"      phone booths\n"
		"answer <person>     buy <item> in stores"
		"save/load           quit\n"
		"say/talk to <person>exit\n"
		"teleport            call\n"
		"\n"
		"Special hints:\n"
		"*  Use maps.  They're not worth points,\n"
		"   but are sprinkled throughout the game"
		"   and can help you navigate through the"
		"   game.\n"
		"*  Scanners allow you to detect nearby\n"
		"   objects.  Just type scan <object>.\n"
		"   You can have multiple scanners.\n"
		);*/
	printh (hidereadw((void*)&HelpPages[1]));
#else
	prints (HelpPages[0]);
	if (getkey()=='x') return;
	printscr (HelpPages[1]);
#endif
	//return;
}


void vEnter ()//char Itm1)
{
	//if (Itm1==iTeleport_) Itm1=iTeleporter;
	switch (Itm1)
	{
	}
	/*printf (Message[msgCantEnter], Item[Itm1].Name);*/
	prints ("Can't enter a ");
	//printh (hidereadw((void*)&Item[Itm1].Name));
	printitmname ();
	printperiod ();
	//return;
}

#ifdef __C128__
#pragma code-name ("TAPECODE")
#endif

void vRead ()//char Itm1)
{
	if (SearchInvPlayer()) {
		if (SearchInvRoom()<0) {
			//printf (Message[msgItemNotAvail]);
			//printmsg (msgItemNotAvail);
			printnoitem ();
			return;
		}
	}
//printh (hidereadw((void*)&Item[Itm1].Name)); printcr ();
	switch (Itm1)
	{
	}
	printscr ("Can't read this.");
}

//#pragma bss-name (push, "V2BSS")

void vSay ()
{
	switch (Itm1)
	{
	case 255:
		printscr ("Say to whom?"); return;
	}
	prints ("You can't talk to ");
	//printh (hidereadw((void*)&Item[Itm1].Name));
	printitmname ();
	printperiod(); 
	//return;
	//}
}

#ifdef __C128__
#pragma code-name (pop)
#endif

//#pragma bss-name (pop)
//#pragma bss-name ("BSS")

